#!/bin/bash

set -e
XRAY_USER_NAME=${XRAY_USER_NAME:-"xray"}
XRAY_USER_GROUP=${XRAY_USER_GROUP:-"xray"}
# PREREQUISITES
DOCKER_SERVER_MIN_VER='1.11'
#DOCKER_SERVER_CURRENT_VER, this should be passed by the installer script [xray.sh] as $(docker version --format '{{.Server.Version}}') || DOCKER_SERVER_CURRENT_VER=""

MAX_OPEN_FILES_MIN_VALUE=10000
MAX_OPEN_FILES_REQUIRED_VALUE=32000

# XRAY_APP_SERVICES_LIST, the services names as they are in the docker-cmpose file, SPACE separated
XRAY_APP_SERVICES_LIST="xray-server xray-indexer xray-analysis xray-event xray-persist"

# PREREQUISITES
DOCKER_SERVER_MIN_VER='1.11'
#DOCKER_SERVER_CURRENT_VER, this should be passed by the installer script [xray.sh] as $(docker version --format '{{.Server.Version}}') || DOCKER_SERVER_CURRENT_VER=""

RECOMMENDED_MIN_RAM=8388608     # it means that for Xray need more than 8G Total RAM => 8*1024*1024k=8388608
RECOMMENDED_MAX_USED_STORAGE=80  # it means that for Xray need more than 20% available storage
RECOMMENDED_MIN_CPU=4
CANCEL_INSTALLER_STATE=false
VALID_USER_CHOICE=false

XRAY_CONFIG_FILE_TEMPLATE="/opt/jfrog/xray/installer/xray_config.yml.template"

INSTALLER_INFO_FILE="/data/installer.info"
DEPLOYED="deployed"
RABBIT_CONNECTION_STRING_DEFAULT='amqp://guest:guest@rabbitmq:5672'
MONGO_CONNECTION_STRING_DEFAULT='mongodb://xray:password@mongodb:27017/?authSource=xray&authMechanism=SCRAM-SHA-1'
POSTGRES_CONNECTION_STRING_DEFAULT='postgres://xray:xray@postgres:5432/xraydb?sslmode=disable'
MONGO_KEY='mongoUrl'
POSTGRES_KEY='postgresqlUrl'
RABBIT_KEY='mqBaseUrl'

aparse()
{
    if [ ! -z $1 ]; then
        command=$1
        shift
    else
        command=usage
        return 0
    fi

    while [[ $# > 0 ]] ; do
      case "$1" in
        --compose-flags)
            shift
              export DOCKER_COMPOSE_FLAGS=${1}
              shift
          ;;
        --server-flags)
              shift
              export DOCKER_COMPOSE_XRAY_SERVER_FLAGS=${1}
              shift
          ;;
        --persist-flags)
              shift
              export DOCKER_COMPOSE_XRAY_PERSIST_FLAGS=${1}
              shift
          ;;
        --indexer-flags)
              shift
              export DOCKER_COMPOSE_XRAY_INDEXER_FLAGS=${1}
              shift
          ;;
        --xevent-flags)
              shift
              export DOCKER_COMPOSE_XRAY_XEVENT_FLAGS=${1}
              shift
          ;;
        --analysis-flags)
              shift
              export DOCKER_COMPOSE_XRAY_ANALYSIS_FLAGS=${1}
              shift
          ;;
          *)
              ALL_ARGS="${ALL_ARGS}${1} "
              shift
          ;;
      esac
    done
}

if [ -z "$COMPOSE_PROJECT_NAME" ]; then
    COMPOSE_PROJECT_NAME="xray"
fi

# Installed version
INSTALLER_DATA_FOLDER="/data"
INSTALLED_VERSION_FILE="${INSTALLER_DATA_FOLDER}/version.current"
INSTALLED_VERSION_FILE_HISTORY="${INSTALLER_DATA_FOLDER}/version.history"
INSTALLED_VERSION=''
EULA_DOC="${INSTALLER_DATA_FOLDER}/JFrog_Xray_EULA.info"
EULA_DOC_SRC="/opt/jfrog/xray/installer/JFrog_Xray_EULA.info"
XRAY_GLOBAL_MOUNT_ROOT="/xray_global_mount_root" # This should not be used, unless special maintenance is needed
XRAY_CONFIG_FILE="${XRAY_GLOBAL_MOUNT_ROOT}/xray/config/xray_config.yaml"
XRAY_CONFIG_DIR=$(dirname "${XRAY_CONFIG_FILE}")
VALUES_YML="/opt/jfrog/xray/installer/values.yml"
DOCKER_COMPOSE_GENERATED_FILE="/opt/jfrog/xray/installer/docker-compose.yml"

if [ -f $INSTALLED_VERSION_FILE ]
then
    INSTALLED_VERSION=$(cat $INSTALLED_VERSION_FILE)
fi

# Maintainer / Installer version
MAINTAINER_VERSION_FILE="/opt/jfrog/xray/installer/version.current"
MAINTAINER_VERSION=''
if [ -f $MAINTAINER_VERSION_FILE ]
then
    MAINTAINER_VERSION=$(cat $MAINTAINER_VERSION_FILE)
fi

#load xray configuration
XRAY_SERVER_PORT_DEFAULT=8000
XRAY_SERVER_PORT=$(./getXrayConfValue.py -f "${XRAY_CONFIG_FILE}" -k "XrayServerPort" 2>/dev/null ) || true #echo "XrayServerPort cannot be loaded from xray_config.yaml. Using default value: ${XRAY_SERVER_PORT_DEFAULT} "
XRAY_SERVER_PORT=${XRAY_SERVER_PORT:-${XRAY_SERVER_PORT_DEFAULT}}
re_is_num='^[0-9]+$'
if ! [[ $XRAY_SERVER_PORT =~ $re_is_num ]] ; then
   echo "ERROR: xray_config.yaml: XrayServerPort is not a number" >&2; exit 1
fi

# Default compose env
export XRAY_SERVER_PORT=${XRAY_SERVER_PORT}
export XRAY_DATA="/var/opt/jfrog/xray/data"
export DOCKER_COMPOSE_FLAGS=''
export DOCKER_COMPOSE_XRAY_SERVER_FLAGS=''
export DOCKER_COMPOSE_XRAY_PERSIST_FLAGS=''
export DOCKER_COMPOSE_XRAY_INDEXER_FLAGS=''
export DOCKER_COMPOSE_XRAY_XEVENT_FLAGS=''
export DOCKER_COMPOSE_XRAY_ANALYSIS_FLAGS=''

exitOnError()
{
    msg=$1
    echo "ERROR: ${msg}"
    exit 1
}

usage() {
    cat ./readme.txt
    exit 1
}

version_lt() { test "$(echo "$@" | tr " " "\n" | sort -rV | head -n 1)" != "$1"; }

# $1 should be WARNING or ERROR
printUlimitErrorMsg()
{
    local error_level=$1
    local number=$2
    MSG="
${error_level}: Max number of open files [${number}] is too low !!!

Add the following to /etc/security/limits.conf file on HOST:
* soft nofile ${MAX_OPEN_FILES_REQUIRED_VALUE}
* hard nofile ${MAX_OPEN_FILES_REQUIRED_VALUE}

Note: Also verify docker limits configuration if exists [usually in /etc/default/docker]

"

    case ${error_level} in
    "ERROR")
        echo -e "\033[31m${MSG}\033[0m "
    ;;
    "WARNING")
        echo -e "\033[33m${MSG}\033[0m"
    ;;
    esac

    echo
}

userChoiceVerification() {
    if [[ "$1" = [nN] ]] || [[ "$1" = [yY] ]]; then
        VALID_USER_CHOICE=true
    else
        echo "Invalid option."
    fi;
}

checkPrerequisites()
{
    #echo "Check dependencies:"
    echo "Verifying Xray prerequisites ..."
    TOTAL_RAM="$(grep ^MemTotal /proc/meminfo | awk '{print $2}')"
    USED_STORAGE="$(df -h /data | tail -n +2 | tr -s ' ' | tr '%' ' ' | cut -d ' ' -f 5)"
    FREE_CPU="$(grep -c ^processor /proc/cpuinfo)"
    local msg=""

    if [ -z ${DOCKER_SERVER_CURRENT_VER} ] || [ "${DOCKER_SERVER_CURRENT_VER}" == "" ]
    then
        msg="WARNING: Docker daemon version is unknown"
        echo -e "\033[33m${msg}\033[0m"
        CANCEL_INSTALLER_STATE=true
        # Do not exit on warning
    fi

    if version_lt ${DOCKER_SERVER_CURRENT_VER} ${DOCKER_SERVER_MIN_VER}
    then
        echo ""
        exitOnError "Xray requires Docker daemon version ${DOCKER_SERVER_MIN_VER} or later. The current version is ${DOCKER_SERVER_CURRENT_VER}."
    fi

    # echo "Check max number of open files:"
    currentMaxOpenFilesVal=$(ulimit -n)

    if [ "${currentMaxOpenFilesVal}" != "unlimited" ] ; then

        # MAX_OPEN_FILES_REQUIRED_VALUE
        if [ ${currentMaxOpenFilesVal} -lt ${MAX_OPEN_FILES_REQUIRED_VALUE} ]
        then
            #echo "Trying to set max open files = ${MAX_OPEN_FILES_REQUIRED_VALUE}"
            ulimit -n ${MAX_OPEN_FILES_REQUIRED_VALUE} 2>/dev/null
            if [ $? -gt 0 ]
            then
                # Cannot set MAX_OPEN_FILES_REQUIRED_VALUE, will try to set MAX_OPEN_FILES_MIN_VALUE
                # MAX_OPEN_FILES_MIN_VALUE
                if [ ${currentMaxOpenFilesVal} -lt ${MAX_OPEN_FILES_MIN_VALUE} ]
                then
                    #echo "Trying to set max open files = ${MAX_OPEN_FILES_MIN_VALUE}"
                    ulimit -n ${MAX_OPEN_FILES_MIN_VALUE} 2>/dev/null
                    if [ $? -gt 0 ]; then
                        printUlimitErrorMsg "ERROR" "${currentMaxOpenFilesVal}"
                        return 1
                    fi
                fi
                printUlimitErrorMsg "WARNING" "${currentMaxOpenFilesVal}"
                CANCEL_INSTALLER_STATE=true
                # Do not exit on warning
            fi
        fi

    fi

    #Checking RAM
    if [[ ${TOTAL_RAM} -lt ${RECOMMENDED_MIN_RAM} ]]; then
        let "TOTAL_RAM_TO_SHOW = ${TOTAL_RAM} / 1024 / 1024"
        msg="WARNING: Running with ${TOTAL_RAM_TO_SHOW}GB Total RAM"
        echo -e "\033[33m${msg}\033[0m"
        CANCEL_INSTALLER_STATE=true
    fi;
    #Checking disk space
    if [[ ${USED_STORAGE} -gt ${RECOMMENDED_MAX_USED_STORAGE} ]]; then
        let "AVAILABLE_STORAGE = 100 - ${USED_STORAGE}"
        msg="WARNING: Running with $AVAILABLE_STORAGE% Free Storage"
        echo -e "\033[33m${msg}\033[0m"
        CANCEL_INSTALLER_STATE=true
    fi;

    #Checking cpu
    if [ ${FREE_CPU} -lt ${RECOMMENDED_MIN_CPU} ]; then
        msg="WARNING: Running with $FREE_CPU CPU Cores"
        echo -e "\033[33m${msg}\033[0m"
        CANCEL_INSTALLER_STATE=true
    fi;

    #Asking user to proceed or not
    if [ ${CANCEL_INSTALLER_STATE} != "false" ]; then
        until [ ${VALID_USER_CHOICE} == true ]; do
             read -p "The System resources are not aligned with Xray minimal perquisites, Do you want to proceed with the installation? [Y/N]: " USER_CHOICE
             userChoiceVerification ${USER_CHOICE}
        done
    fi;

    #In case that user typed "nN", exiting"
    if [[ "${USER_CHOICE}" =~ [nN] ]]; then
        echo "Exiting..."
        exit 1
    fi;

    # dependencies check is passed
    return 0
}

# Creating Xray config dir and setting xray user permissions on files
setupDirsPermissions () {

    local installer_mount_root="${INSTALLER_DATA_FOLDER}"
    local xray_mount_root="${XRAY_GLOBAL_MOUNT_ROOT}/xray"
    echo "Checking permissions on ${XRAY_MOUNT_ROOT} on host"
    chown -R ${XRAY_USER_NAME}:${XRAY_USER_GROUP} ${installer_mount_root} || exitOnError "Setting ownership on ${XRAY_MOUNT_ROOT}/xray-installer : ${installer_mount_root} (on the host : inside the container) failed "

    # xray_global_mount_root in the container is equivalent to XRAY_MOUNT_ROOT on the host
    STAT=( $(stat -c "%U %G" ${xray_mount_root}) )
    USER=${STAT[0]}
    GROUP=${STAT[1]}

    if [[ ${USER} != "$XRAY_USER_NAME" ]] || [[ ${GROUP} != "$XRAY_USER_GROUP"  ]] ; then
         echo "$XRAY_MOUNT_ROOT is owned by $USER:$GROUP. Setting to $XRAY_USER_NAME:$XRAY_USER_GROUP."
         echo "NOTE: The following procedures change the ownership of files and may take several minutes. Do not stop the installation/upgrade process."
         chown -R ${XRAY_USER_NAME}:${XRAY_USER_GROUP} ${xray_mount_root} || exitOnError "Setting ownership on ${XRAY_MOUNT_ROOT}/xray : ${xray_mount_root} (on the host : inside the container) failed "
    else
        echo "$XRAY_MOUNT_ROOT is already owned by $XRAY_USER_NAME:$XRAY_USER_GROUP."
    fi
}

runDockerCommand() {
    local action=$1

    case $action in
    "pull")
       if  [ "$DOCKER_INSTALLER_OFFLINE" == "true" ]; then
            return 0;
       fi
    ;;
    "logs")
        action="logs -f"
    ;;
    "stopApp")
        # Stop Xray App layer
        docker-compose -p ${COMPOSE_PROJECT_NAME} "stop" ${XRAY_APP_SERVICES_LIST}
        return $?
    ;;
    "restart")
        docker-compose -p ${COMPOSE_PROJECT_NAME} "stop"
        sleep 3
        action="up -d"
    ;;
    "restartApp")
        # Restart Xray App Layer
        docker-compose -p ${COMPOSE_PROJECT_NAME} "stop" ${XRAY_APP_SERVICES_LIST}
        sleep 3
        action="up -d"
    ;;
    esac

    docker-compose -p ${COMPOSE_PROJECT_NAME} ${action}
}

# Returned code:
#0) $1=$2
#1) $1>$2
#2) $1<$2
verComp () {
    if [[ $1 == $2 ]]
    then
        return 0
    fi
    local IFS=.
    local i
    local ver1=($1)
    local ver2=($2)
    # fill empty fields in ver1 with zeros
    for ((i=${#ver1[@]}; i<${#ver2[@]}; i++))
    do
        ver1[i]=0
    done
    for ((i=0; i<${#ver1[@]}; i++))
    do
        if [[ -z ${ver2[i]} ]]
        then
            # fill empty fields in ver2 with zeros
            ver2[i]=0
        fi
        if ((10#${ver1[i]} > 10#${ver2[i]}))
        then
            return 1
        fi
        if ((10#${ver1[i]} < 10#${ver2[i]}))
        then
            return 2
        fi
    done
    return 0
}

isUpgradeAllowed() {

    local installedVersion=$INSTALLED_VERSION
    local maintainerVersion=$MAINTAINER_VERSION

    if [ -z $MAINTAINER_VERSION ]
    then
        exitOnError "MAINTAINER_VERSION is unknown"
    fi

    # This is a fresh installation
    if [ -z $INSTALLED_VERSION ]
    then
        return 0
    fi

    if [[ "$installedVersion" =~ ^b.* ]] && [[ "$maintainerVersion" =~ ^b.* ]]
    then
        installedVersion=($(echo $INSTALLED_VERSION | sed "s/^b//g"))
        maintainerVersion=($(echo $MAINTAINER_VERSION | sed "s/^b//g"))
    fi

    verComp $installedVersion $maintainerVersion 2>/dev/null
    if [ $? -eq 1 ]
    then
        return 1
    fi

    return 0
}

isCommandAllowed()
{
    local command=$1
    local installedVersion=$INSTALLED_VERSION
    local maintainerVersion=$MAINTAINER_VERSION

    case ${command} in
        "install")
            [ -f $INSTALLED_VERSION_FILE ] && exitOnError "Xray is already installed. To upgrade, use: ./xray upgrade"
            checkPrerequisites || exitOnError "Prerequisites are not satisfied"
        ;;
        "upgrade")
            [ -f $INSTALLED_VERSION_FILE ] || exitOnError "Xray is not installed. To install, use: ./xray install"
            checkPrerequisites || exitOnError "Prerequisites are not satisfied"
            isUpgradeAllowed || exitOnError "Unable to upgrade to an older version"
        ;;
        "version")

        ;;
        "info")

        ;;
        *)
            # Is installed
            [ -f $INSTALLED_VERSION_FILE ] || exitOnError "Xray is not installed. To install, use: ./xray install"
            # Is correct version
            [ "${installedVersion}" == "${maintainerVersion}" ] || exitOnError "You are using the Xray installer version [${maintainerVersion}] which is different from the currently installed version [${installedVersion}]. Consider upgrading your version of Xray using: ./xray upgrade"
        ;;

    esac

    return 0
}

setVersions(){

    if [ ! -f ${MAINTAINER_VERSION_FILE} ]
    then
        return 1
    fi

    cat ${MAINTAINER_VERSION_FILE} > ${INSTALLED_VERSION_FILE}
    cat ${MAINTAINER_VERSION_FILE} >> ${INSTALLED_VERSION_FILE_HISTORY}

    echo "${INSTALLED_VERSION_FILE}: $(cat ${INSTALLED_VERSION_FILE})"

    return 0
}

setEULA(){
    cp ${EULA_DOC_SRC} ${EULA_DOC}
    return $?
}

createMongoUsers(){
    if [ -f ${INSTALLER_INFO_FILE} ]; then
        source ${INSTALLER_INFO_FILE}
    fi
    if [[ "${INSTALL_MONGO}" = [yY] ]]; then
        USERS_CREATED_FILE="${INSTALLER_DATA_FOLDER}/mongo_users.created"
        if [ ! -f "${USERS_CREATED_FILE}" ];then
          sleep 1
          cat createMongoUsers.js | docker exec -i "${COMPOSE_PROJECT_NAME}_mongodb_1" mongo  > /dev/null && \
          touch $USERS_CREATED_FILE || exitOnError "Failed to create MongoDB default users"
        fi
    fi
}

# backward compatible issues
# Xray should be stopped before this process
backwardCmpatibleProcess()
{
    #echo -e "\nINFO: Backward compatible process"
    LOGS_FOLDER_V1="${XRAY_GLOBAL_MOUNT_ROOT}/xray/log"
    LOGS_FOLDER_V2="${XRAY_GLOBAL_MOUNT_ROOT}/xray/logs"
    CONFIG_FOLDER="${XRAY_GLOBAL_MOUNT_ROOT}/xray/config"
    ARCHIVE_FOLDER="${INSTALLER_DATA_FOLDER}/archive"
    TIMESTAMP=$(date +"%s")

    # Rename log folder to be logs
    if [ -d "${LOGS_FOLDER_V1}" ] && [ ! -d "${LOGS_FOLDER_V2}" ]
    then
        mv ${LOGS_FOLDER_V1} ${LOGS_FOLDER_V2} && \
        echo "Logs folder renamed was:[${LOGS_FOLDER_V1}] new:[${LOGS_FOLDER_V2}]"

        # Archive old log4go_config.xml
        if [ ! -d ${ARCHIVE_FOLDER} ]; then
            mkdir -p ${ARCHIVE_FOLDER} && \
            echo "Archive folder was created [${ARCHIVE_FOLDER}]"
        fi

        for log4goService in "event" "analysis" "indexer" "persist" "server"
        do
           echo "Archive ${log4goService}_log4go_config.xml to ${ARCHIVE_FOLDER}"
           if [ -f "${CONFIG_FOLDER}/${log4goService}_log4go_config.xml" ] && [ -d "${ARCHIVE_FOLDER}" ]
           then
                mv "${CONFIG_FOLDER}/${log4goService}_log4go_config.xml" "${ARCHIVE_FOLDER}/${log4goService}_log4go_config.xml.${TIMESTAMP}"
           fi
        done
    fi
}

#This function is generating "installer.info" file
getThirdpMetadata() {
    if [ ! -f ${XRAY_CONFIG_FILE} ]; then
        #RABBIT
        VALID_USER_CHOICE=false
        until [ ${VALID_USER_CHOICE} == true ]; do
            read -p "Would you like to install RabbitMQ instance? [Y/n]: " INSTALL_RABBIT
            userChoiceVerification ${INSTALL_RABBIT}
        done
        if [[ "${INSTALL_RABBIT}" = [nN] ]]; then
            read -p "Type a RabbitMQ connection string [${RABBIT_CONNECTION_STRING_DEFAULT}]: " RABBIT_CONNECTION_STRING
            RABBIT_CONNECTION_STRING=${RABBIT_CONNECTION_STRING:-${RABBIT_CONNECTION_STRING_DEFAULT}}
        fi
        #POSTGRES
        VALID_USER_CHOICE=false
        until [ ${VALID_USER_CHOICE} == true ]; do
            read -p "Would you like to install PostgreSQL instance? [Y/n]: " INSTALL_POSTGRES
            userChoiceVerification ${INSTALL_POSTGRES}
        done
        if [[ "${INSTALL_POSTGRES}" = [nN] ]]; then
            read -p "Type a PostgreSQL connection string [${POSTGRES_CONNECTION_STRING_DEFAULT}]: " POSTGRES_CONNECTION_STRING
            POSTGRES_CONNECTION_STRING=${POSTGRES_CONNECTION_STRING:-${POSTGRES_CONNECTION_STRING_DEFAULT}}
        fi
        #MONGO
        VALID_USER_CHOICE=false
        until [ ${VALID_USER_CHOICE} == true ]; do
            read -p "Would you like to install MongoDB instance? [Y/n]: " INSTALL_MONGO
            userChoiceVerification ${INSTALL_MONGO}
        done
        if [[ "${INSTALL_MONGO}" = [nN] ]]; then
            read -p "Type a MongoDB connection string [${MONGO_CONNECTION_STRING_DEFAULT}]: " MONGO_CONNECTION_STRING
            MONGO_CONNECTION_STRING=${MONGO_CONNECTION_STRING:-${MONGO_CONNECTION_STRING_DEFAULT}}
        fi
        #SETTING VARS IN installer.info
        if [ ! -f ${INSTALLER_INFO_FILE} ]; then
            touch ${INSTALLER_INFO_FILE}
        fi

        sed -i '/.*INSTALL_RABBIT.*/d' ${INSTALLER_INFO_FILE} || echo "There is nothing to delete, skipping ..."
        echo "export INSTALL_RABBIT=${INSTALL_RABBIT}" >> ${INSTALLER_INFO_FILE}

        sed -i '/.*RABBIT_CONNECTION_STRING.*/d' ${INSTALLER_INFO_FILE} || echo "There is nothing to delete, skipping ..."
        echo "export RABBIT_CONNECTION_STRING=${RABBIT_CONNECTION_STRING}" >> ${INSTALLER_INFO_FILE}

        sed -i '/.*INSTALL_POSTGRES.*/d' ${INSTALLER_INFO_FILE} || echo "There is nothing to delete, skipping ..."
        echo "export INSTALL_POSTGRES=${INSTALL_POSTGRES}" >> ${INSTALLER_INFO_FILE}

        sed -i '/.*POSTGRES_CONNECTION_STRING.*/d' ${INSTALLER_INFO_FILE} || echo "There is nothing to delete, skipping ..."
        echo "export POSTGRES_CONNECTION_STRING=${POSTGRES_CONNECTION_STRING}" >> ${INSTALLER_INFO_FILE}

        sed -i '/.*INSTALL_MONGO.*/d' ${INSTALLER_INFO_FILE} || echo "There is nothing to delete, skipping ..."
        echo "export INSTALL_MONGO=${INSTALL_MONGO}" >> ${INSTALLER_INFO_FILE}

        sed -i '/.*MONGO_CONNECTION_STRING.*/d' ${INSTALLER_INFO_FILE} || echo "There is nothing to delete, skipping ..."
        echo "export MONGO_CONNECTION_STRING=${MONGO_CONNECTION_STRING}" >> ${INSTALLER_INFO_FILE}
    fi
}

#This function replaces values in xray_conf.yml template file
replaceStringByKey() {
    key=$1
    connection_string=$2
    value=$(grep ${key} ${XRAY_CONFIG_FILE} | awk '{print $2}')

    #Replacing & with %% in order to prevent sed issues
    connection_string=$(echo ${connection_string} | sed -e "s|\&|%%|g")
    value=$(echo ${value} | sed -e "s|\&|%%|g")
    sed -e "s|\&|%%|g" -i ${XRAY_CONFIG_FILE}

    #Where choose a non default app AND connection string is still default
    if [ ! "${value}" == "${DEPLOYED}" ]; then
        sed -e "s|${value}|${connection_string}|g" -i ${XRAY_CONFIG_FILE}

    #Replacing %% with & (REVERT)
        sed -e "s|%%|\&|g" -i ${XRAY_CONFIG_FILE}
        connection_string=$(echo ${connection_string} | sed -e "s|%%|\&|g")
        sed --in-place "s|${connection_string}|${DEPLOYED}|g" ${INSTALLER_INFO_FILE}
    fi
}

#This function is generating "xray_conf.yml" file
setValuesXrayConfig() {
    if [ -f ${INSTALLER_INFO_FILE} ]; then
        source ${INSTALLER_INFO_FILE}
    fi
    if [ ! -f ${XRAY_CONFIG_FILE} ]; then
        if [ ! -d ${XRAY_CONFIG_DIR} ] ; then
            mkdir -p ${XRAY_CONFIG_DIR}
        fi
        cp ${XRAY_CONFIG_FILE_TEMPLATE} ${XRAY_CONFIG_FILE}

        if [[ "${INSTALL_RABBIT}" = [nN] ]]; then
            replaceStringByKey ${RABBIT_KEY} ${RABBIT_CONNECTION_STRING}
        fi
        if [[ "${INSTALL_POSTGRES}" = [nN] ]]; then
            replaceStringByKey ${POSTGRES_KEY} ${POSTGRES_CONNECTION_STRING}
        fi
        if [[ "${INSTALL_MONGO}" = [nN] ]]; then
            replaceStringByKey ${MONGO_KEY} ${MONGO_CONNECTION_STRING}
        fi
    fi
}

#This function is generating docker-compose.yml and values.yml
generateComposeFile() {
    if [ -f ${DOCKER_COMPOSE_GENERATED_FILE} ]; then
        rm -rf ${DOCKER_COMPOSE_GENERATED_FILE}
    fi

    #Generating values.yml
    if [[ "${INSTALL_RABBIT}" = [nN] ]]; then
        sed -i '/.*rabbitmq.*/d' ${VALUES_YML} || echo "There is nothing to delete, skipping ..."
        echo "  rabbitmq: false" >> ${VALUES_YML}
    fi
    if [[ "${INSTALL_POSTGRES}" = [nN] ]]; then
        sed -i '/.*postgres.*/d' ${VALUES_YML} || echo "There is nothing to delete, skipping ..."
        echo "  postgres: false" >> ${VALUES_YML}
    fi
    if [[ "${INSTALL_MONGO}" = [nN] ]]; then
        sed -i '/.*mongodb.*/d' ${VALUES_YML} || echo "There is nothing to delete, skipping ..."
        echo "  mongodb: false" >> ${VALUES_YML}
    fi

    #Gdenerating docker-compose.yml
    python generate-compose-file.py
    if [ $? -gt 0 ]; then
        echo "Error while generating a customized docker-compose.yml, Exiting..."
        return 1
    fi
}

##################################
############## MAIN ##############
##################################

aparse $@

isCommandAllowed ${command}

case $command in
    install)
        ( getThirdpMetadata && \
          setValuesXrayConfig && \
          generateComposeFile && \
          runDockerCommand pull && \
          setVersions && \
          setEULA \
          setupDirsPermissions \
        ) || exitOnError "Installation failed"
        echo
        echo -e "\033[32mINFO: Xray $MAINTAINER_VERSION was successfully installed. Run: ./xray start\033[0m"
        echo "To access Xray, browse to http://<server_name>:8000"
        echo "For more information, please refer to the JFrog Xray User Guide: https://www.jfrog.com/confluence/display/XRAY/Welcome+to+JFrog+Xray"
        echo
        echo -e "\033[33mNote: You may want to make 'xray' the owner of Xray files. In order to do that please run the following command on the host:\033[0m"
        echo -e "\033[33mgroupadd -g 1035 xray && useradd -u 1035 -g 1035 -m -s /bin/false xray\033[0m"
        ;;
    upgrade)
        ( getThirdpMetadata && \
          setValuesXrayConfig && \
          generateComposeFile && \
          runDockerCommand pull && \
          runDockerCommand stopApp && \
          backwardCmpatibleProcess && \
          setVersions && \
          setupDirsPermissions \
        ) || exitOnError "Upgrade failed"
        echo
        echo "INFO: Xray was successfully upgraded. Run: ./xray start"
        echo ""
        ;;
    start)
        getThirdpMetadata && \
        setValuesXrayConfig && \
        generateComposeFile && \
        runDockerCommand "up -d" && \
        createMongoUsers && \
        setupDirsPermissions
        ;;
    ps|logs|stop|kill|restart|restartApp)
        getThirdpMetadata && \
        setValuesXrayConfig && \
        generateComposeFile && \
        runDockerCommand ${command}
        ;;
    x-rm)
        getThirdpMetadata && \
        setValuesXrayConfig && \
        generateComposeFile && \
        runDockerCommand rm
        ;;
    version)
        if [ -f $INSTALLED_VERSION_FILE ]; then
            echo "Xray version: ${INSTALLED_VERSION}"
        else
            echo "Xray is not installed"
        fi
        ;;
    x-compose)
            xComposeCmd="docker-compose -p ${COMPOSE_PROJECT_NAME} ${ALL_ARGS}"
            echo "Advanced user: Going to execute '${xComposeCmd}'"
            ${xComposeCmd}
        ;;
    info)
           echo "Xray info:"
           echo "server port: ${XRAY_SERVER_PORT}"
           echo "project name: ${COMPOSE_PROJECT_NAME}"
           echo "Xray docker repo: ${XRAY_DOCKER_REPO}"
           echo "Xray DB docker repo: ${XRAYDB_DOCKER_REPO}"
           echo "docker mount folder: ${XRAY_MOUNT_ROOT}"
        ;;
    *)
        usage
        ;;
esac
